/*
 * Copyright (c)1997 by Dianne Hackborn.
 * All rights reserved.
 *
 * Based (increasingly loosly) on WebTerm, a Web-based terminal
 * applet written in Java, which is
 * Copyright (C)1996 by the National Alliance for Computational
 * Science and Engineering (NACSE).
 * See <URL:http://www.nacse.org/> for more information.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Dianne Hackborn,
 * at <hackbod@lucent.com> or <hackbod@enteract.com>.
 *
 * ----------------------------------------------------------------------
 *
 * ArpEmulatorInterface.h
 *
 * The ArpEmulator class defines an abstract interface to a
 * terminal emulation.  It is used by the ArpTerminal class
 * to transform incoming text/control characters into their
 * appropriate terminal operations, and transform user input
 * into a proper outgoing text/control character stream.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * To Do
 * ~~~~~
 *
 *	• Need to define an interface for allowing the emulator
 *	  to interact with the user to change its settings.
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * 7/19/97:
 *	Created file from ArpEmulator.h
 *
 */

#pragma once

#ifndef ARPCOMMON_ARPEMULATORINTERFACE_H
#define ARPCOMMON_ARPEMULATORINTERFACE_H

#ifndef ARPCOMMON_ARPTERMINALINTERFACE_H
#include <ArpCommon/ArpTerminalInterface.h>
#endif

#ifndef	_ARCHIVABLE_H
#include <support/Archivable.h>
#endif

struct ArpEmulationType {
	int32 StructLength;			// sizeof(ArpEmulationType)
	const char* Name;
	const char* LongName;
	const char* Synonym;
};

class ArpEmulatorInterface : public BArchivable {

public:

	/* ------------------------------------------------------------
	   BARCHIVABLE INTERFACE
	   ------------------------------------------------------------ */

	virtual void AttachTerminal(ArpTerminalInterface& terminal) = 0;
	
	/* ------------------------------------------------------------
	   GLOBAL CONTROL METHODS
	   ------------------------------------------------------------ */

	// Reset all emulation state to its start-up settings, which
	// should be some sane user-friendly values.  The emulator
	// should also call the terminal's TermReset() method as part
	// of this operation.
	virtual void Reset(bool hard) = 0;

	// Retrieve the type information about this emulator object.
	virtual const ArpEmulationType* EmulationType(void) const = 0;
	
	/* ------------------------------------------------------------
	   FUNCTIONS FOR MOVING DATA THROUGH THE EMULATION
	   ------------------------------------------------------------ */

	// Transform a character stream into the terminal operations
	// needed to display the text to the user.  The emulator
	// displays the final text to the user through the terminal's
	// TermSendTTY() and other functions.
	virtual void EmulateToTTY(const ichar* d, size_t len) = 0;

	// Transform user input to a character stream, which should
	// be sent to the remote device that is attached to this
	// terminal.  The emulator sends the final character stream
	// to the remote device through the terminal's
	// TermSendRemote() function.  The second function's return
	// value indicates whether the terminal actually did something
	// with the message -- if you return false, the terminal will
	// be able to do its own processing on the message, for
	// example use a mouse down to start highlighting.
	virtual void EmulateToRemote(const ichar* d, size_t len) = 0;
	virtual bool EmulateToRemote(BMessage* msg) = 0;

};

#endif
